/*****************************************

ECWorkshop -- evolutionary computation
toolkit library

Web: http://qai.narod.ru/ecw/
Mailto: qai@mail.ru

Copyright (C) 2007 by Yury Tsoy


This program is free software; you can
redistribute it and/or modify it under the
terms of the GNU General Public License as
published by the Free Software Foundation;
either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that
it will be useful, but WITHOUT ANY WARRANTY;
without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR
PURPOSE. See the GNU General Public License
for more details.

You should have received a copy of the GNU
General Public License along with this
program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330,
Boston, MA  02111-1307  USA

*/

/* TODO (Qai#1#): Support of both increasing and decreasing fitness 
                  (maximization and minimization problems respectively) */

#include "problems/include.h"
#include "eas/include.h"
#include "processors/include.h"
#include "../qtools/qlightxml.h"

#include "operators/include.h"

#include <process.h>
#include <stdio.h>

using namespace std;
using namespace ECWorkshop;

QProblem* createProblem (QLightXMLIO& config);
QEvolutionaryAlgorithm* createEA (QLightXMLIO& config);

int main(int argc, char* argv[]) {

	QLightXMLIO xmlio;
	QEAParameters params;
	int runs = 100;

	if (argc > 1) {
		xmlio.readFromFile (argv[1]);
		params.readFromXML (xmlio);
	} else {
		xmlio.readFromFile ("config.xml");
		params.readFromXML (xmlio);
	}
	if (argc > 2) {
		runs = atoi (argv[2]);
	} else {
		runs = 1;
	}

	QEvolutionaryAlgorithm *ea = createEA (xmlio);	//new QGeneticAlgorithm ();
	QProblem *problem = NULL;

	params.ea = ea;

	problem = createProblem (xmlio);
	problem->printProblemName (cout);

	params.problem = problem;
	problem->getDefaultParameters (&params);
	problem->setParameters (&params);
	ea->setParameters (params);

	QSummaryProcessor sumProcessor;
	sumProcessor.setParameters (&params);
	for (int i=0; i<runs; i++) {
		printf ("Run # %i\n", i);
		ea->run ();
		sumProcessor.addSummary (ea->getSummary());
	}
	sumProcessor.operate ();

	ea->finishWork ();

	delete problem;
	delete ea;

	return 0;
}

QProblem* createProblem (QLightXMLIO& config) {
	QXMLTAG_LIST tags;
	string strTemp;

	config.getTagByName ("problem", tags);
	strTemp = tags[0]._tags[0]._contents;	// read contents of the first embedded tag
	if (strTemp == "Sphere") {
		return new QSphereFunction ();
	} else if (strTemp == "Rastrigin") {
		return new QRastriginFunction ();
	} else if (strTemp == "Schwefel") {
		return new QSchwefelFunction ();
	} else if (strTemp == "Rosenbrock") {
		return new QRosenbrockFunction ();
	} else if (strTemp == "Deceptive") {
		return new QDeceptiveProblem ();
	} else if (strTemp == "Mixing-time") {
		return new QMixingTimeResearch ();
	} else if (strTemp == "Onemax") {
		return new QOneMaxProblem ();
	} else if (strTemp == "Pointless") {
		return new QPointlessProblem ();
	} else if (strTemp == "Submatrix") {
		return new QSubmatrixTask ();
	} else if (strTemp == "XOR") {
		return new QANNProblemXOR ();
	} else if (strTemp == "Double sin") {
		return new QANNProblemSinDouble ();
	} else {
		printf ("\nOopsie, no problem!\n");
		exit (0);
	}
}

QEvolutionaryAlgorithm* createEA (QLightXMLIO& config) {
	QXMLTAG_LIST tags;
	string strTemp;

	config.getTagByName ("algorithm", tags);
	strTemp = tags[0]._tags[0]._contents;	// read contents of the first embedded tag
	if (strTemp == "GA") {
		cout<<"Creating GA\n";
		return new QGeneticAlgorithm ();
	} else if (strTemp == "SARN") {
		cout<<"Creating QSARN\n";
		return new QSARN ();
	} else if (strTemp == "SARN2") {
		cout<<"Creating QSARN2 (indirect evolution)\n";
		return new QSARN2 ();
	} else {
		printf ("\nIs it true? No algorithm?!\n");
		exit (0);
	}
}