/*****************************************

ECWorkshop -- evolutionary computation
toolkit library

Web: http://qai.narod.ru/ecw/
Mailto: qai@mail.ru

Copyright (C) 2007 by Yury Tsoy


This program is free software; you can
redistribute it and/or modify it under the
terms of the GNU General Public License as
published by the Free Software Foundation;
either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that
it will be useful, but WITHOUT ANY WARRANTY;
without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR
PURPOSE. See the GNU General Public License
for more details.

You should have received a copy of the GNU
General Public License along with this
program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330,
Boston, MA  02111-1307  USA

*/

#include "qdistance.h"

#include "include.h"
#include "../include/qec.h"

using namespace ECWorkshop;

///////////////////////////////////////////////
//
//	QDistance class
//
///////////////////////////////////////////////

// protected methods
//
double QDistance::measureHamming (QIndividual* argInd1, QIndividual* argInd2) {
	// check sizes
	if (argInd1->getSize() <= 0 || argInd2->getSize() <=0) {
		cerr<<"\n\n[QDistance::measureHamming] error:\n"
			<<"\tIncorrect chromosome size! ("<<argInd1->getSize()<<" : "<<argInd2->getSize()<<")\n\n";
		exit (0);
	}

	QEncodingType encoding = argInd1->getEncodingType();
	int i;
	int dist = 0, size = argInd1->getSize ();

	if (encoding == QET_INTEGER) {
		// [integer] encoding
		//
		QIntegerIndividual *ind1 = (QIntegerIndividual *)argInd1, *ind2 = (QIntegerIndividual *)argInd2;
		int temp, geneSize = ind1->getGeneSize ();
		int j;

		for (i=0; i<size; i++) {
			temp = ind1->getGeneValue (i) ^ ind2->getGeneValue(i);
			// calculate units
			for (j=0; j<geneSize; j++) {
				if (temp & bitMask[j]) {
					dist++;
				}
			}
		}

	} else {
		// [string] or [permutation] encoding
		//
		for (i=0; i<size; i++) {
			if (!argInd1->getGene(i)->equalsTo (argInd2->getGene(i))) {
				dist++;
			}
		}
	}	// if (encoding == QET_INTEGER)

	return dist;
}

double QDistance::measureEuclidean (QIndividual* argInd1, QIndividual* argInd2) {
	return 0;
}

// public methods
//
QDistance::QDistance () {}
QDistance::~QDistance () {}

double QDistance::measure (QIndividual* argInd1, QIndividual* argInd2) {
	if (argInd1->getEncodingType() != argInd2->getEncodingType()) {
		cerr<<"\n\n[QDistance::measure] error:\n"
			<<"\tDifferent encodings for compared chromosomes! ("<<argInd1->getEncodingType()<<" : "<<argInd2->getEncodingType()<<")\n\n";
		exit (0);
	}
	switch (argInd1->getEncodingType()) {
		case QET_INTEGER:
		case QET_STRING:
		case QET_PERMUTATION:
			return measureHamming (argInd1, argInd2);
			break;
		case QET_REAL:
			return measureEuclidean (argInd1, argInd2);
			break;
		default:
			return -1.0;
	}
}
