/*****************************************

ECWorkshop -- evolutionary computation
toolkit library

Web: http://qai.narod.ru/ecw/
Mailto: qai@mail.ru

Copyright (C) 2007 by Yury Tsoy


This program is free software; you can
redistribute it and/or modify it under the
terms of the GNU General Public License as
published by the Free Software Foundation;
either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that
it will be useful, but WITHOUT ANY WARRANTY;
without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR
PURPOSE. See the GNU General Public License
for more details.

You should have received a copy of the GNU
General Public License along with this
program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330,
Boston, MA  02111-1307  USA

*/

#include "qstringgene.h"

using namespace ECWorkshop;

///////////////////////////////////////////////////////////
//
//		QStringGene class
//
///////////////////////////////////////////////////////////

QStringGene::QStringGene (void)
: _value(0) {
}

QStringGene::QStringGene (const QStringGene& argGene)
: QGene(argGene), _value(argGene._value) {
}

QStringGene::~QStringGene (void) {
	_value = 0;
}

QStringGene& QStringGene::operator= (const QStringGene& argGene) {
	QGene::operator = (argGene);
	_value = argGene._value;
	return (*this);
}

QGene* QStringGene::clone () const {
	return new QStringGene (*this);
}

QEncodingType QStringGene::getEncodingType () const {
	return QET_STRING;
}

bool QStringGene::equalsTo (const QGene* argGene) const {
	if (argGene->getEncodingType() == QET_STRING) {
		unsigned int temp = ((QStringGene*)argGene)->_value;
		return (_value == temp)? true : false;
	} else {
		cerr<<"\n\n[QStringGene::equalsTo] error:\n"
			<<"\tCan't compare genes with different encodings\n\n";
		return false;
	}
}

int QStringGene::assign (const QGene* argGene) {
	if (argGene->getEncodingType() == QET_STRING) {
		_value = ((QStringGene*)argGene)->_value;
		return Q_OK;
	} else {
		cerr<<"\n\n[QStringGene::assign] error:\n"
			<<"\tEncoding mismatch! (encoding ID: "<<argGene->getEncodingType()<<")\n\n";
		return Q_FAILURE;
	}
}

unsigned int QStringGene::getValue () const {
	return _value;
}

int QStringGene::setValue (const unsigned int argValue) {
	_value = argValue;
	return Q_OK;
}
