/*****************************************

ECWorkshop -- evolutionary computation
toolkit library

Web: http://qai.narod.ru/ecw/
Mailto: qai@mail.ru

Copyright (C) 2007 by Yury Tsoy


This program is free software; you can
redistribute it and/or modify it under the
terms of the GNU General Public License as
published by the Free Software Foundation;
either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that
it will be useful, but WITHOUT ANY WARRANTY;
without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR
PURPOSE. See the GNU General Public License
for more details.

You should have received a copy of the GNU
General Public License along with this
program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330,
Boston, MA  02111-1307  USA

*/


#include "qec.h"

#include "../eas/include.h"
#include "../problems/include.h"

#include <time.h>

using namespace ECWorkshop;

///////////////////////////////////////////////////////////
//
//	QGAParameters structure
//
///////////////////////////////////////////////////////////

QEAParameters::QEAParameters () : chromosomeLength(10), populationSize (50), generationsNumber (500),
arity(2), geneSize(10), geneOffset (-5.12), genePrecision(0.01), initializationType(QIT_RANDOM),
tournamentSize (2), eliteCount(2), parentsNumber(2), childrenNumber(2), xRate (0.7), mRate (0.01),
errorLevel(0.01), problem(NULL), encoding(QET_UNKNOWN), selectionType(QST_UNKNOWN), parentSelectorType(QPST_UNKNOWN),
xType (QXT_2POINT), mutationType(QMT_BITFLIP), populationSizerType(QPSZT_CONST),
minPopSize(10), maxPopSize(200), sizingDirection(0), sizingStep(0), unitsRatio(0.5), permutation(false),
order (QO_UNKNOWN), removeClones(false), eaType(QEAT_GENETIC_ALGORITHM), populationsNumber(1),
nichingType (QNT_UNKNOWN), sharingSigma(0.0), inputs(0), outputs(0), untilFirstHit (false), ea(NULL) {
}

QEAParameters::~QEAParameters () {
	populationsNumber = 0;
	chromosomeLength = 0;
	arity = 0;
	geneSize = 0;
	geneOffset = 0;
	genePrecision = 0;
	populationSize = 0;
	generationsNumber = 0;
	tournamentSize = 0;
	eliteCount = 0;
	parentsNumber = 0;
	childrenNumber = 0;
	xRate = 0.0;
	mRate = 0.0;
	unitsRatio = 0.0;
	errorLevel = 0;
	minPopSize = maxPopSize = 0;
	permutation = false;
	untilFirstHit = false;
	sharingSigma = 0.0;

	eaType = QEAT_UNKNOWN;
	encoding = QET_UNKNOWN;
	initializationType = QIT_UNKNOWN;
	selectionType = QST_UNKNOWN;
	parentSelectorType = QPST_UNKNOWN;
	nichingType = QNT_UNKNOWN;
	populationSizerType = QPSZT_UNKNOWN;
	xType = QXT_UNKNOWN;
	mutationType = QMT_UNKNOWN;

	problem = NULL;
	ea = NULL;
}

int QEAParameters::print2stream (ostream& out) const {
	out<<"EA parameters:\n"
		<<"---------------------------\n";
	out<<"Algorithm's name:\t";
	ea->printName(out);
	out<<"Problem:\t";
	problem->printProblemName(out);
	out<<"EA type ID:\t"<<eaType<<"\n"
		<<"populations number:\t"<<populationsNumber<<"\n"
		<<"chromosomeLength:\t"<<chromosomeLength<<"\n"
		<<"arity:\t"<<arity<<"\n"
		<<"geneSize:\t"<<geneSize<<"\n"
		<<"geneOffset:\t"<<geneOffset<<"\n"
		<<"genePrecision:\t"<<genePrecision<<"\n"
		<<"populationSize:\t"<<populationSize<<"\n"
		<<"generationsNumber:\t"<<generationsNumber<<"\n"
		<<"tournamentSize:\t"<<tournamentSize<<"\n"
		<<"eliteCount:\t"<<eliteCount<<"\n"
		<<"parentsNumber:\t"<<parentsNumber<<"\n"
		<<"childrenNumber:\t"<<childrenNumber<<"\n"
		<<"xRate:\t"<<xRate<<"\n"
		<<"mRate:\t"<<mRate<<"\n"
		<<"errorLevel:\t"<<errorLevel<<"\n"
		<<"encoding ID:\t"<<encoding<<"\n"
		<<"permutation:\t"<<permutation<<"\n"
		<<"encoding order ID:\t"<<order<<"\n"
		<<"initializationType ID:\t"<<initializationType<<"\n"
		<<"units ratio:\t"<<unitsRatio<<"\n"
		<<"nichingType ID:\t"<<nichingType<<"\n"
		<<"sharing sigma:\t"<<sharingSigma<<"\n"
		<<"selectionType ID:\t"<<selectionType<<"\n"
		<<"parentSelectorType ID:\t"<<parentSelectorType<<"\n"
		<<"populationSizerType ID:\t"<<populationSizerType<<"\n"
		<<"min allowed population size:\t"<<minPopSize<<"\n"
		<<"max allowed population size:\t"<<maxPopSize<<"\n"
		<<"duplicate chromosomes removal:\t"<<removeClones<<"\n"
		<<"xType ID:\t"<<xType<<"\n"
		<<"mutationType ID:\t"<<mutationType<<"\n"
		<<"RNG seed:\t"<<seed<<"\n"
		<<"Until first hit:\t"<<untilFirstHit<<"\n";

	out<<"\n";

	return Q_OK;
}

int QEAParameters::readFromXML (const QLightXMLIO& xml) {
	QXMLTAG_LIST names, values;
	string strTemp;

	xml.getTagByName ("name", names);
	xml.getTagByName ("value", values);

	QXMLTAG_LIST::iterator iter = names.begin(), end = names.end();
	for (; iter != end; iter++) {
		//-----------------------
		if ((*iter)._contents.compare ("Arity") == 0) {
			char2int (arity, values[iter-names.begin()]._contents.c_str());
			continue;
		}
		//-----------------------
		if ((*iter)._contents.compare ("Children number") == 0) {
			char2int (childrenNumber, values[iter-names.begin()]._contents.c_str());
			continue;
		}
		//-----------------------
		if ((*iter)._contents.compare ("Chromosome length") == 0) {
			char2int (chromosomeLength, values[iter-names.begin()]._contents.c_str());
			continue;
		}
		//-----------------------
		if ((*iter)._contents.compare ("Elite count") == 0) {
			char2int (eliteCount, values[iter-names.begin()]._contents.c_str());
			continue;
		}
		//-----------------------
		if ((*iter)._contents.compare ("Encoding") == 0) {
			strTemp = values[iter-names.begin()]._contents;
			if (strTemp == "integer") {
				encoding = QET_INTEGER;
			} else if (strTemp == "string") {
				encoding = QET_STRING;
			} else if (strTemp == "real") {
				encoding = QET_REAL;
			} else if (strTemp == "permutation") {
				encoding = QET_STRING;
				permutation = true;
			} else if (strTemp == "regulatory network") {
				encoding = QET_REGULATORY_NETWORK;
				permutation = true;
			}
			continue;
		}
		//-----------------------
		if ((*iter)._contents.compare ("Error level") == 0) {
			char2dbl (errorLevel, values[iter-names.begin()]._contents.c_str());
			continue;
		}
		//-----------------------
		if ((*iter)._contents.compare ("Gene offset") == 0) {
			char2dbl (geneOffset, values[iter-names.begin()]._contents.c_str());
			continue;
		}
		//-----------------------
		if ((*iter)._contents.compare ("Gene precision") == 0) {
			char2dbl (genePrecision, values[iter-names.begin()]._contents.c_str());
			continue;
		}
		//-----------------------
		if ((*iter)._contents.compare ("Generations number") == 0) {
			char2int (generationsNumber, values[iter-names.begin()]._contents.c_str());
			continue;
		}
		//-----------------------
		if ((*iter)._contents.compare ("Gene size") == 0) {
			char2int (geneSize, values[iter-names.begin()]._contents.c_str());
			continue;
		}
		//-----------------------
		if ((*iter)._contents.compare ("Initialization type") == 0) {
			strTemp = values[iter-names.begin()]._contents;
			if (strTemp == "random") {
				initializationType = QIT_RANDOM;
			} else if (strTemp == "unmixed") {
				initializationType = QIT_UNMIXED;
			}
			continue;
		}
		//-----------------------
		if ((*iter)._contents.compare ("Units ratio") == 0) {
			char2dbl (unitsRatio, values[iter-names.begin()]._contents.c_str());
			continue;
		}
		//-----------------------
		if ((*iter)._contents.compare ("Max population size") == 0) {
			char2int (maxPopSize, values[iter-names.begin()]._contents.c_str());
			continue;
		}
		//-----------------------
		if ((*iter)._contents.compare ("Min population size") == 0) {
			char2int (minPopSize, values[iter-names.begin()]._contents.c_str());
			continue;
		}
		//-----------------------
		if ((*iter)._contents.compare ("Mutation rate") == 0) {
			char2dbl (mRate, values[iter-names.begin()]._contents.c_str());
			continue;
		}
		//-----------------------
		if ((*iter)._contents.compare ("Mutation type") == 0) {
			strTemp = values[iter-names.begin()]._contents;
			if (strTemp == "bit-flip") {
				mutationType = QMT_BITFLIP;
			} else if (strTemp == "random flip") {
				mutationType = QMT_RANDOMFLIP;
			} else if (strTemp == "gaussian") {
				mutationType = QMT_GAUSS;
			} else if (strTemp == "basic") {
				mutationType = QMT_RN_BASIC;
			}
			continue;
		}
		//-----------------------
		if ((*iter)._contents.compare ("Niching type") == 0) {
			strTemp = values[iter-names.begin()]._contents;
			if (strTemp == "no niching") {
				nichingType = QNT_UNKNOWN;
			} else if (strTemp == "sharing") {
				nichingType = QNT_SHARING;
			} else if (strTemp == "deterministic crowding") {
				nichingType = QNT_CROWDING_DETERMINISTIC;
			}
			continue;
		}
		//-----------------------
		if ((*iter)._contents.compare ("Encoding order") == 0) {
			strTemp = values[iter-names.begin()]._contents;
			if (strTemp == "no order") {
				order = QO_UNKNOWN;
			} else if (strTemp == "ascending") {
				order = QO_ASCENDING;
			} else if (strTemp == "descending") {
				order = QO_DESCENDING;
			}
			continue;
		}
		//-----------------------
		if ((*iter)._contents.compare ("Parents number") == 0) {
			char2int (parentsNumber, values[iter-names.begin()]._contents.c_str());
			continue;
		}
		//-----------------------
		if ((*iter)._contents.compare ("Parents selection type") == 0) {
			strTemp = values[iter-names.begin()]._contents;
			if (strTemp == "random") {
				parentSelectorType = QPST_RANDOM;
			}
			continue;
		}
		//-----------------------
		if ((*iter)._contents.compare ("Population size") == 0) {
			char2int (populationSize, values[iter-names.begin()]._contents.c_str());
			continue;
		}
		//-----------------------
		if ((*iter)._contents.compare ("Population sizer type") == 0) {
			strTemp = values[iter-names.begin()]._contents;
			if (strTemp == "constant") {
				populationSizerType = QPSZT_CONST;
			} else if (strTemp == "steady") {
				populationSizerType = QPSZT_STEADY;
			} else if (strTemp == "simple") {
				populationSizerType = QPSZT_SIMPLE;
			} else if (strTemp == "simple Fibonacci") {
				populationSizerType = QPSZT_SIMPLE_FIBONACCI;
			}
			continue;
		}
		//-----------------------
		if ((*iter)._contents.compare ("Remove clones") == 0) {
			strTemp = values[iter-names.begin()]._contents;
			if (strTemp == "yes") {
				removeClones = true;
			} else if (strTemp == "no") {
				removeClones = false;
			}
			continue;
		}
		//-----------------------
		if ((*iter)._contents.compare ("RNG seed") == 0) {
			int temp;
			char2int (temp, values[iter-names.begin()]._contents.c_str());
			seed = temp;
			if (seed == 0) {
				seed = (unsigned)time(NULL);
			}
			continue;
		}
		//-----------------------
		if ((*iter)._contents.compare ("Selection type") == 0) {
			strTemp = values[iter-names.begin()]._contents;
			if (strTemp == "neutral") {
				selectionType = QST_NEUTRAL;
			} else if (strTemp == "tournament") {
				selectionType = QST_TOURNAMENT;
			}
			continue;
		}
		//-----------------------
		if ((*iter)._contents.compare ("Sharing sigma") == 0) {
			char2dbl (sharingSigma, values[iter-names.begin()]._contents.c_str());
			continue;
		}
		//-----------------------
		if ((*iter)._contents.compare ("Sizing direction") == 0) {
			strTemp = values[iter-names.begin()]._contents;
			if (strTemp == "up") {
				sizingDirection = 1;
			} else if (strTemp == "down") {
				sizingDirection = -1;
			} else if (strTemp == "no") {
				sizingDirection = 0;
			}
			continue;
		}
		//-----------------------
		if ((*iter)._contents.compare ("Sizing step") == 0) {
			char2int (sizingStep, values[iter-names.begin()]._contents.c_str());
			continue;
		}
		//-----------------------
		if ((*iter)._contents.compare ("Tournament size") == 0) {
			char2int (tournamentSize, values[iter-names.begin()]._contents.c_str());
			continue;
		}
		//-----------------------
		if ((*iter)._contents.compare ("Crossover rate") == 0) {
			char2dbl (xRate, values[iter-names.begin()]._contents.c_str());
			continue;
		}
		//-----------------------
		if ((*iter)._contents.compare ("Crossover type") == 0) {
			strTemp = values[iter-names.begin()]._contents;
			if (strTemp == "1-point") {
				xType = QXT_1POINT;
			} else if (strTemp == "2-point") {
				xType = QXT_2POINT;
			} else if (strTemp == "uniform") {
				xType = QXT_UNIFORM;
			} else if (strTemp == "1-point PGX") {
				xType = QXT_1POINT_PGX;
			} else if (strTemp == "2-point PGX") {
				xType = QXT_2POINT_PGX;
			} else if (strTemp == "arithmetic") {
				xType = QXT_ARITHMETIC;
			} else if (strTemp == "BLX") {
				xType = QXT_BLX;
			} else if (strTemp == "SBX") {
				xType = QXT_SBX;
			} else if (strTemp == "order") {
				xType = QXT_ORDER;
			}
			continue;
		}
		//-----------------------
		if ((*iter)._contents.compare ("xAlpha") == 0) {
			char2dbl (xAlpha, values[iter-names.begin()]._contents.c_str());
			continue;
		}
		//-----------------------
		if ((*iter)._contents.compare ("Until first hit") == 0) {
			strTemp = values[iter-names.begin()]._contents;
			if (strTemp == "yes") {
				untilFirstHit = true;
			} else if (strTemp == "no") {
				untilFirstHit = false;
			}
			continue;
		}
	}	// for (; iter != end; iter++)

	return Q_OK;
}


///////////////////////////////////////////////////////////
//
//	QFitness class
//
///////////////////////////////////////////////////////////

QFitness::QFitness () : _value(0) {
}

QFitness::QFitness (const QFitness& argFitness) : _value( argFitness._value ) {
	_extra = argFitness._extra;
}

QFitness::~QFitness () {
	_value = 0;
	_extra.clear ();
}

QFitness& QFitness::operator= (const QFitness& argFitness) {
	_value = argFitness._value;
	_extra = argFitness._extra;

	return (*this);
}

QFitness& QFitness::operator= (const double argD) {
	_value = argD;
	_extra.clear ();

	return (*this);
}

double QFitness::getValue () const {
	return _value;
}

int QFitness::setValue (const double argValue) {
	_value = argValue;
	return Q_OK;
}

double QFitness::getExtra (const int argIndex) const {
	return _extra[argIndex];
}

int QFitness::setExtra (const int argIndex, const double argValue) {
	_extra[argIndex] = argValue;
	return Q_OK;
}

int QFitness::addExtra (const double argValue) {
	_extra.push_back (argValue);
	return Q_OK;
}

int QFitness::print2Stream (ostream& out) const {
	int i, size;

	out<<_value;
	if (!_extra.empty()) {
		size = (int)_extra.size ();
		out<<" ";
		for (i=0; i<size; ++i) {
			out<<"["<<_extra[i]<<"]";
		}
	}
	return Q_OK;
}
