/*****************************************

ECWorkshop -- evolutionary computation
toolkit library

Web: http://qai.narod.ru/ecw/
Mailto: qai@mail.ru

Copyright (C) 2007 by Yury Tsoy


This program is free software; you can
redistribute it and/or modify it under the
terms of the GNU General Public License as
published by the Free Software Foundation;
either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that
it will be useful, but WITHOUT ANY WARRANTY;
without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR
PURPOSE. See the GNU General Public License
for more details.

You should have received a copy of the GNU
General Public License along with this
program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330,
Boston, MA  02111-1307  USA

*/

#include "qrunsummary.h"
#include "../encodings/include.h"

using namespace ECWorkshop;

///////////////////////////////////////////////////////////
//
//	QRunSummary structure
//
///////////////////////////////////////////////////////////

QRunSummary::QRunSummary () : feCount(0), firstHitFECount(0), firstHitGeneration(0),
generation(0), time(0.0), firstHitTime(0.0), startClock (0), finishClock(0), firstHitClock(0),
currentBest(NULL), firstHitSolution(NULL), convergenceGeneration(0) {
	means.clear ();
	devs.clear ();
	bests.clear ();
	worsts.clear ();
	popSizes.clear ();
}

QRunSummary::QRunSummary (const QRunSummary& argSummary)
: feCount(argSummary.feCount), firstHitFECount(argSummary.firstHitFECount),
firstHitGeneration(argSummary.firstHitGeneration), generation(argSummary.generation),
time(argSummary.time), firstHitTime(argSummary.firstHitTime),
startClock (argSummary.startClock), finishClock(argSummary.finishClock),
firstHitClock(argSummary.firstHitClock), convergenceGeneration(argSummary.convergenceGeneration) {

	currentBest = argSummary.currentBest->clone ();
	if (argSummary.firstHitSolution != NULL) {
		firstHitSolution = argSummary.firstHitSolution->clone ();
	} else {
		firstHitSolution = NULL;
	}
	means.assign (argSummary.means.begin(), argSummary.means.end());
	devs.assign (argSummary.devs.begin(), argSummary.devs.end());
	bests.assign (argSummary.bests.begin(), argSummary.bests.end());
	worsts.assign (argSummary.worsts.begin(), argSummary.worsts.end());
	popSizes.assign (argSummary.popSizes.begin(), argSummary.popSizes.end());
}

QRunSummary::~QRunSummary () {
	reset ();
}

QRunSummary& QRunSummary::operator= (const QRunSummary& argSummary) {

	feCount = argSummary.feCount;
	generation = argSummary.generation;
	time = argSummary.time;
	startClock = argSummary.startClock;
	finishClock = argSummary.finishClock;
	firstHitFECount = argSummary.firstHitFECount;
    firstHitGeneration = argSummary.firstHitGeneration;
	firstHitClock = argSummary.firstHitClock;
	firstHitTime = argSummary.firstHitTime;
	convergenceGeneration = argSummary.convergenceGeneration;

	if (currentBest != NULL) {
		delete currentBest;
		currentBest = NULL;
	}
	if (argSummary.currentBest != NULL) {
		currentBest = argSummary.currentBest->clone ();
	}

	if (firstHitSolution != NULL) {
		delete firstHitSolution;
		firstHitSolution = NULL;
	}
	if (argSummary.firstHitSolution != NULL) {
		firstHitSolution = argSummary.firstHitSolution->clone ();
	}

	means.assign (argSummary.means.begin(), argSummary.means.end());
	devs.assign (argSummary.devs.begin(), argSummary.devs.end());
	bests.assign (argSummary.bests.begin(), argSummary.bests.end());
	worsts.assign (argSummary.worsts.begin(), argSummary.worsts.end());
	popSizes.assign (argSummary.popSizes.begin(), argSummary.popSizes.end());

	return (*this);
}

int QRunSummary::reset () {
	generation = 0;
	feCount = 0;
	firstHitFECount = 0;
	firstHitGeneration = 0;
	time = 0.0;
	firstHitTime = 0.0;
	startClock = 0;
	finishClock = 0;
	firstHitClock = 0;
	convergenceGeneration = 0;

//	cout<<means.size()<<"\n";
//	cout.flush ();
	means.clear ();
	devs.clear ();
	bests.clear ();
	worsts.clear ();
	popSizes.clear ();

	if (currentBest != NULL) {
		delete currentBest;
		currentBest = NULL;
	}

	if (firstHitSolution != NULL) {
		delete firstHitSolution;
		firstHitSolution = NULL;
	}

	return Q_OK;
}

