/*****************************************

ECWorkshop -- evolutionary computation
toolkit library

Web: http://qai.narod.ru/ecw/
Mailto: qai@mail.ru

Copyright (C) 2007 by Yury Tsoy


This program is free software; you can
redistribute it and/or modify it under the
terms of the GNU General Public License as
published by the Free Software Foundation;
either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that
it will be useful, but WITHOUT ANY WARRANTY;
without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR
PURPOSE. See the GNU General Public License
for more details.

You should have received a copy of the GNU
General Public License along with this
program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330,
Boston, MA  02111-1307  USA

*/

#ifndef		Q_INTEGER_CROSSOVER_OPERATOR
#define		Q_INTEGER_CROSSOVER_OPERATOR

#include "qcrossover.h"

namespace ECWorkshop {

///////////////////////////////////////////////////////////
//
//		QIntegerCrossover class
//
///////////////////////////////////////////////////////////

class QIntegerCrossover : public QCrossoverOperator {
protected:
	QIntegerCrossover (void);

public:
	QIntegerCrossover (const QEvolutionaryAlgorithm* owner);
	QIntegerCrossover (const QIntegerCrossover& argX);
	virtual ~QIntegerCrossover (void);

	virtual QIntegerCrossover& operator= (const QCrossoverOperator& argX);

	virtual QEncodingType getEncodingType () const;
};


///////////////////////////////////////////////////////////
//
//		QIntegerCrossover1 class
//
// Notes:
// --------------------------
// 1-point crossover operator for integer encoding
//
//
// !!! Important !!!
// bits per gene as well as number of genes in each chromosome are
// supposed to be equal over the whole population and constant
//
///////////////////////////////////////////////////////////

class QIntegerCrossover1 : public QIntegerCrossover {
protected:

	virtual int cross (const __VECTOR_CONST_INDIVIDUAL_PTR* argParents, __VECTOR_INDIVIDUAL_PTR* children);

	QIntegerCrossover1 (void);

public:
	QIntegerCrossover1 (const QEvolutionaryAlgorithm* owner);
	QIntegerCrossover1 (const QIntegerCrossover1& argX);
	virtual ~QIntegerCrossover1 (void);

	virtual QIntegerCrossover1& operator= (const QIntegerCrossover1& argX);

	virtual QCrossoverOperator* clone () const;
};


///////////////////////////////////////////////////////////
//
//		QIntegerCrossover2 class
//
// Notes:
// --------------------------
// 2-point crossover operator for integer encoding
//
//
// !!! Important !!!
// bits per gene as well as number of genes in each chromosome are
// supposed to be equal over the whole population and constant
//
///////////////////////////////////////////////////////////

class QIntegerCrossover2 : public QIntegerCrossover {
protected:

	virtual int cross (const __VECTOR_CONST_INDIVIDUAL_PTR* argParents, __VECTOR_INDIVIDUAL_PTR* children);

	QIntegerCrossover2 (void);

public:
	QIntegerCrossover2 (const QEvolutionaryAlgorithm* owner);
	QIntegerCrossover2 (const QIntegerCrossover2& argX);
	virtual ~QIntegerCrossover2 (void);

	virtual QIntegerCrossover2& operator= (const QIntegerCrossover2& argX);

	virtual QCrossoverOperator* clone () const;
};


///////////////////////////////////////////////////////////
//
//		QIntegerCrossoverUniform class
//
// Notes:
// --------------------------
// Uniform crossover operator for [integer] encoding
//
//
// !!! Important !!!
// bits per gene as well as number of genes in each chromosome are
// supposed to be equal over the whole population and constant
//
///////////////////////////////////////////////////////////

class QIntegerCrossoverUniform : public QIntegerCrossover {
protected:

	virtual int cross (const __VECTOR_CONST_INDIVIDUAL_PTR* argParents, __VECTOR_INDIVIDUAL_PTR* children);

	QIntegerCrossoverUniform (void);

public:
	QIntegerCrossoverUniform (const QEvolutionaryAlgorithm* owner);
	QIntegerCrossoverUniform (const QIntegerCrossoverUniform& argX);
	virtual ~QIntegerCrossoverUniform (void);

	virtual QIntegerCrossoverUniform& operator= (const QIntegerCrossoverUniform& argX);

	virtual QCrossoverOperator* clone () const;
};


///////////////////////////////////////////////////////////
//
//		QIntegerCrossoverPGX1 class
//
// Notes:
// --------------------------
// 1-point per-gene crossover operator for integer encoding
//
//
// !!! Important !!!
// bits per gene as well as number of genes in each chromosome are
// supposed to be equal over the whole population and constant
//
///////////////////////////////////////////////////////////

class QIntegerCrossoverPGX1 : public QIntegerCrossover {
protected:

	virtual int cross (const __VECTOR_CONST_INDIVIDUAL_PTR* argParents, __VECTOR_INDIVIDUAL_PTR* children);

	QIntegerCrossoverPGX1 (void);

public:
	QIntegerCrossoverPGX1 (const QEvolutionaryAlgorithm* owner);
	QIntegerCrossoverPGX1 (const QIntegerCrossoverPGX1& argX);
	virtual ~QIntegerCrossoverPGX1 (void);

	virtual QIntegerCrossoverPGX1& operator= (const QIntegerCrossoverPGX1& argX);

	virtual QCrossoverOperator* clone () const;
};


///////////////////////////////////////////////////////////
//
//		QIntegerCrossoverPGX2 class
//
// Notes:
// --------------------------
// 2-point per-gene crossover operator for integer encoding
//
//
// !!! Important !!!
// bits per gene as well as number of genes in each chromosome are
// supposed to be equal over the whole population and constant
//
///////////////////////////////////////////////////////////

class QIntegerCrossoverPGX2 : public QIntegerCrossover {
protected:

	virtual int cross (const __VECTOR_CONST_INDIVIDUAL_PTR* argParents, __VECTOR_INDIVIDUAL_PTR* children);

	QIntegerCrossoverPGX2 (void);

public:
	QIntegerCrossoverPGX2 (const QEvolutionaryAlgorithm* owner);
	QIntegerCrossoverPGX2 (const QIntegerCrossoverPGX2& argX);
	virtual ~QIntegerCrossoverPGX2 (void);

	virtual QIntegerCrossoverPGX2& operator= (const QIntegerCrossoverPGX2& argX);

	virtual QCrossoverOperator* clone () const;
};

};	// namespace

#endif


