/*****************************************

ECWorkshop -- evolutionary computation
toolkit library

Web: http://qai.narod.ru/ecw/
Mailto: qai@mail.ru

Copyright (C) 2007 by Yury Tsoy


This program is free software; you can
redistribute it and/or modify it under the
terms of the GNU General Public License as
published by the Free Software Foundation;
either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that
it will be useful, but WITHOUT ANY WARRANTY;
without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR
PURPOSE. See the GNU General Public License
for more details.

You should have received a copy of the GNU
General Public License along with this
program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330,
Boston, MA  02111-1307  USA

*/

#ifndef		Q_STRING_CROSSOVER_OPERATOR
#define		Q_STRING_CROSSOVER_OPERATOR

#include "qcrossover.h"

namespace ECWorkshop {

///////////////////////////////////////////////////////////
//
//		QStringCrossover class
//
///////////////////////////////////////////////////////////

class QStringCrossover : public QCrossoverOperator {
protected:
	QStringCrossover (void);

public:
	QStringCrossover (const QEvolutionaryAlgorithm* owner);
	QStringCrossover (const QStringCrossover& argX);
	virtual ~QStringCrossover (void);

	virtual QStringCrossover& operator= (const QStringCrossover& argX);

	virtual QEncodingType getEncodingType () const;
};


///////////////////////////////////////////////////////////
//
//		QStringCrossover1 class
//
// Notes:
// --------------------------
// 1-point crossover operator for [string] encoding
//
//
// !!! Important !!!
// bits per gene as well as number of genes in each chromosome are
// supposed to be equal over the whole population and constant
//
///////////////////////////////////////////////////////////

class QStringCrossover1 : public QStringCrossover {
protected:

	virtual int cross (const __VECTOR_CONST_INDIVIDUAL_PTR* argParents, __VECTOR_INDIVIDUAL_PTR* children);

	QStringCrossover1 (void);

public:
	QStringCrossover1 (const QEvolutionaryAlgorithm* owner);
	QStringCrossover1 (const QStringCrossover1& argX);
	virtual ~QStringCrossover1 (void);

	virtual QStringCrossover1& operator= (const QStringCrossover1& argX);

	virtual QCrossoverOperator* clone () const;
};


///////////////////////////////////////////////////////////
//
//		QStringCrossover2 class
//
// Notes:
// --------------------------
// 2-point crossover operator for [string] encoding
//
//
// !!! Important !!!
// bits per gene as well as number of genes in each chromosome are
// supposed to be equal over the whole population and constant
//
///////////////////////////////////////////////////////////

class QStringCrossover2 : public QStringCrossover {
protected:

	virtual int cross (const __VECTOR_CONST_INDIVIDUAL_PTR* argParents, __VECTOR_INDIVIDUAL_PTR* children);

	QStringCrossover2 (void);

public:
	QStringCrossover2 (const QEvolutionaryAlgorithm* owner);
	QStringCrossover2 (const QStringCrossover2& argX);
	virtual ~QStringCrossover2 (void);

	virtual QStringCrossover2& operator= (const QStringCrossover2& argX);

	virtual QCrossoverOperator* clone () const;
};


///////////////////////////////////////////////////////////
//
//		QStringCrossoverUniform class
//
// Notes:
// --------------------------
// Uniform crossover operator for [string] encoding
//
//
// !!! Important !!!
// bits per gene as well as number of genes in each chromosome are
// supposed to be equal over the whole population and constant
//
///////////////////////////////////////////////////////////

class QStringCrossoverUniform : public QStringCrossover {
protected:

	virtual int cross (const __VECTOR_CONST_INDIVIDUAL_PTR* argParents, __VECTOR_INDIVIDUAL_PTR* children);

	QStringCrossoverUniform (void);

public:
	QStringCrossoverUniform (const QEvolutionaryAlgorithm* owner);
	QStringCrossoverUniform (const QStringCrossoverUniform& argX);
	virtual ~QStringCrossoverUniform (void);

	virtual QStringCrossoverUniform& operator= (const QStringCrossoverUniform& argX);

	virtual QCrossoverOperator* clone () const;
};

///////////////////////////////////////////////////////////
//
//		QStringCrossoverOrder class
//
// Notes:
// --------------------------
// Order crossover operator for [string] encoding. Should be
//  used for permutation encodings
//
//
// !!! Important !!!
// bits per gene as well as number of genes in each chromosome are
// supposed to be equal over the whole population and constant
//
///////////////////////////////////////////////////////////

class QStringCrossoverOrder : public QStringCrossover {
protected:

	bool hasGene (const QIndividual*, const QGene*, const int bound);
	virtual int cross (const __VECTOR_CONST_INDIVIDUAL_PTR* argParents, __VECTOR_INDIVIDUAL_PTR* children);

	QStringCrossoverOrder (void);

public:
	QStringCrossoverOrder (const QEvolutionaryAlgorithm* owner);
	QStringCrossoverOrder (const QStringCrossoverOrder& argX);
	virtual ~QStringCrossoverOrder (void);

	virtual QStringCrossoverOrder& operator= (const QStringCrossoverOrder& argX);

	virtual QCrossoverOperator* clone () const;
};

};	// namespace

#endif



