/*****************************************

ECWorkshop -- evolutionary computation
toolkit library

Web: http://qai.narod.ru/ecw/
Mailto: qai@mail.ru

Copyright (C) 2007 by Yury Tsoy


This program is free software; you can
redistribute it and/or modify it under the
terms of the GNU General Public License as
published by the Free Software Foundation;
either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that
it will be useful, but WITHOUT ANY WARRANTY;
without even the implied warranty of
MERNHANTABILITY or FITNESS FOR A PARTICULAR
PURPOSE. See the GNU General Public License
for more details.

You should have received a copy of the GNU
General Public License along with this
program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330,
Boston, MA  02111-1307  USA

*/

#include "qrninitializer.h"
#include "../../encodings/include.h"
#include "../mutation/qrnmutation.h"

using namespace ECWorkshop;

///////////////////////////////////////////////////////////
//
//		QRNInitializer class
//
///////////////////////////////////////////////////////////


int QRNInitializer::useRandom () {
	int i, j, k;
	int popSize = _popul->getSize (), chrSize;
	QRNIndividual *tempInd = NULL;
	QRNGene *tempGene = NULL;
	QSignal tempSignal;

//	cout<<"pop size = "<<popSize<<"\n";
	for (i=0; i<popSize; i++) {

		tempInd = ( QRNIndividual* )_popul->getIndividual(i);
		chrSize = tempInd->getSize ();
//		cout<<"chrSize = "<<chrSize<<"\n";

		for (j=0; j<chrSize; j++) {
			tempGene = ( QRNGene* )tempInd->getGene(j);

			// set number of inputs and outputs
			tempGene->setInputsNumber( _params->inputs );
			tempGene->setOutputsNumber( _params->outputs );

			// set random input signal from range [-0.1; +0.1] for the node
			for (k=1; k<=_params->inputs; k++) {
				tempSignal._id = k;
				tempSignal._amount = 0.0;
				tempGene->setInput (k-1, tempSignal);
				// set random input weight
				tempGene->setInputWeight (k-1, 0.2*QRandN() - 0.1);
			}
			// set output signals for the node
			for (; k<=(_params->inputs + _params->outputs); k++) {
//				cout<<"k = "<<k<<"\n";
				tempSignal._id = k;
				tempSignal._amount = 0.0;
				tempGene->setOutput ((k - _params->inputs-1), tempSignal);
				// set output weight
				tempGene->setOutputWeight ((k - _params->inputs-1), 1.0);
			}
			tempGene->setBias (0.2*QRandN() - 0.1);
			tempGene->setParamA (10.0*QRandGauss());
		}

		// set individual's random mutation parameters
		for (j=0; j<MUTATION_PARAMETERS; j++) {
			tempInd->setMutationParam (j, 0.2*QRandN() - 0.1);
		}

	}	//	for (i=0; i<popSize; i++)

	return Q_OK;
}

int QRNInitializer::useUnmixed () {
	// Not supported. Uses random initialization
	return useRandom ();
}

QRNInitializer::QRNInitializer (void) {
}

QRNInitializer::QRNInitializer (const QEvolutionaryAlgorithm* owner)
: QInitializer(owner) {
}

QRNInitializer::QRNInitializer (const QRNInitializer& argInit)
: QInitializer(argInit) {
}

QRNInitializer::~QRNInitializer (void) {
}

int QRNInitializer::operate () {
//	assert (_popul != NULL);

	if (_popul->getEncodingType() == QET_REGULATORY_NETWORK) {

		if (_params->inputs <= 0 || _params->outputs <= 0) {
			cout<<"[QRNInitializer::operate] critical error:\n\t"
				<<"Number of inputs and outputs should be >= 0!\n";
			exit (0);
		}

		switch (_params->initializationType) {
			case QIT_RANDOM:
				useRandom ();
				break;
			default:
				cerr<<"\n\n[QRNInitializer::operate] error:\n"
					<<"\tUnknown initialization type encountered! Using random initialization!\n"
					<<"(initialization type ID = "<<_params->initializationType<<")\n\n";
				useRandom ();
				return Q_FAILURE;
		}

		return Q_OK;

	} else {

		cerr<<"\n\n[QRNInitializer::operate] error:\n"
			<<"\tEncoding mismatch! (population encoding ID = "<<_popul->getEncodingType()<<")\n\n";
		exit (0);

	}	// if (_popul->getEncodingType() == QET_INTEGER)
}

QInitializer* QRNInitializer::clone () const {
	return (new QRNInitializer (*this));
}

QEncodingType QRNInitializer::getEncoding () const {
	return QET_REAL;
}

