/*****************************************

ECWorkshop -- evolutionary computation
toolkit library

Web: http://qai.narod.ru/ecw/
Mailto: qai@mail.ru

Copyright (C) 2007 by Yury Tsoy


This program is free software; you can
redistribute it and/or modify it under the
terms of the GNU General Public License as
published by the Free Software Foundation;
either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that
it will be useful, but WITHOUT ANY WARRANTY;
without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR
PURPOSE. See the GNU General Public License
for more details.

You should have received a copy of the GNU
General Public License along with this
program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330,
Boston, MA  02111-1307  USA

*/

#ifndef		Q_STRING_MUTATION_OPERATOR_H
#define		Q_STRING_MUTATION_OPERATOR_H

#include "qmutation.h"

namespace ECWorkshop {

///////////////////////////////////////////////////////////
//
//		QStringMutation class
//
// Note:
// ------------
// Base class for mutation operators for [string] encoding
//
///////////////////////////////////////////////////////////

class QStringMutation : public QMutationOperator {
protected:
	QStringMutation (void);

public:
	QStringMutation (const QEvolutionaryAlgorithm* owner);
	QStringMutation (const QStringMutation& argMut);
	virtual ~QStringMutation (void);

	QStringMutation& operator= (const QStringMutation& argMut);

	virtual QEncodingType getEncodingType () const;
};

///////////////////////////////////////////////////////////
//
//		QStringMutationRandomFlip class
//
// Note:
// ------------
// Simple random-flip mutation operator for integer encoding
//
///////////////////////////////////////////////////////////

class QStringMutationRandomFlip : public QStringMutation {
protected:
	QStringMutationRandomFlip (void);

public:
	QStringMutationRandomFlip (const QEvolutionaryAlgorithm* owner);
	QStringMutationRandomFlip (const QStringMutationRandomFlip& argMut);
	virtual ~QStringMutationRandomFlip (void);

	virtual QStringMutationRandomFlip& operator= (const QStringMutationRandomFlip& argMut);

	virtual int operate ();
	virtual QMutationOperator* clone () const;
};

};	// namespace

#endif


