/*****************************************

ECWorkshop -- evolutionary computation
toolkit library

Web: http://qai.narod.ru/ecw/
Mailto: qai@mail.ru

Copyright (C) 2007 by Yury Tsoy


This program is free software; you can
redistribute it and/or modify it under the
terms of the GNU General Public License as
published by the Free Software Foundation;
either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that
it will be useful, but WITHOUT ANY WARRANTY;
without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR
PURPOSE. See the GNU General Public License
for more details.

You should have received a copy of the GNU
General Public License along with this
program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330,
Boston, MA  02111-1307  USA

*/

#ifndef	Q_CROWDING
#define	Q_CROWDING

#include "qnichingoperator.h"

#include "../../encodings/include.h"

namespace ECWorkshop {

/////////////////////////////////////////////////////////////
//
//	QCrowding class
//
/////////////////////////////////////////////////////////////

class QCrowding : public QNichingOperator {
protected:
	int _crowdingFactor;	// see K. De Jong's dissertation (p. 138) for details

	QCrowding ();

public:
	QCrowding (const QEvolutionaryAlgorithm* owner);
	QCrowding (const QCrowding& argC);
	virtual ~QCrowding ();

	virtual QCrowding& operator= (const QCrowding& argNiching);

	virtual int setCrowdingFactor (const int argCF);
	virtual int getCrowdingFactor ();

	virtual QNichingOperator* clone ();
	virtual int operate ();
};

/////////////////////////////////////////////////////////////
//
//	QDeterministicCrowding class
//
/////////////////////////////////////////////////////////////

class QDeterministicCrowding : public QNichingOperator {
protected:
	QIndividual *_parent1, *_parent2;
	QIndividual *_child1, *_child2;

	QDeterministicCrowding ();

public:
	QDeterministicCrowding (const QEvolutionaryAlgorithm* owner);
	QDeterministicCrowding (const QDeterministicCrowding& argC);
	virtual ~QDeterministicCrowding ();

	virtual QDeterministicCrowding& operator= (const QDeterministicCrowding& argNiching);

	int setParents (QIndividual* argP1, QIndividual* argP2);
	int setChildren (QIndividual* argC1, QIndividual* argC2);

	virtual QNichingOperator* clone ();
	virtual int operate ();
};

};

#endif
