/*****************************************

ECWorkshop -- evolutionary computation
toolkit library

Web: http://qai.narod.ru/ecw/
Mailto: qai@mail.ru

Copyright (C) 2007 by Yury Tsoy


This program is free software; you can
redistribute it and/or modify it under the
terms of the GNU General Public License as
published by the Free Software Foundation;
either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that
it will be useful, but WITHOUT ANY WARRANTY;
without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR
PURPOSE. See the GNU General Public License
for more details.

You should have received a copy of the GNU
General Public License along with this
program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330,
Boston, MA  02111-1307  USA

*/

#include "qnichingoperator.h"

using namespace ECWorkshop;

///////////////////////////////////////////////////////////
//
//		QNichingOperator class
//
///////////////////////////////////////////////////////////

// int assignNiches ()
// Assignes niches to individuals of the [_popul] population
int QNichingOperator::assignNiches (__TABLE_DOUBLE& mdist) {
	int i, size = _popul->getSize();
	int id = 1, nicheSize = 1;
	__VECTOR_INDIVIDUAL_PTR inds;
	__VECTOR_DOUBLE dists;
	QDistance dist;

	for (i=0; i<size; i++) {
		inds[i] = _popul->getIndividual (i);
	}

	// TODO: Do this to the end, pal!

	dists.resize (size);
	dists[0] = 0;
	for (i=1; i<size; i++) {
		dists[i] = dist.measure (inds[0], inds[1]);
	}

	// assign niches IDs
	for (i=0; i<size; i++) {
		if (inds[i]->getNicheID() == 0) {
			inds[i]->setNicheID (id);
			// TODO: set the same niche id to all individuals similar to given one
			id++;
		}
	}

	return Q_OK;
}

// int calculateDistanceMatrix ()
// Calculates matrix of distances for the [_popul] population
// and writes results into the given matrix
int QNichingOperator::calculateDistanceMatrix (__TABLE_DOUBLE& mdist) {
	int size = _popul->getSize();
	int i, j;
	QDistance dist;
	QIndividual* tempInd;

	mdist.resize (size);
	mdist.assign (size, 0);
	for (i=0; i<size; i++) {
		mdist[i].resize (size);
		mdist[i].assign (size, 0);
	}

	// calculate temporal similarity matrix using _sigma as threshold distance
	for (i=0; i<(size-1); i++) {
		tempInd = _popul->getIndividual(i);
		for (j=(i+1); j<size; j++) {
			//cout<<i<<" : "<<j<<"\n";
			//cout<<dist.measure (tempInd, _popul->getIndividual(j))<<"\n";
			//tempInd->print2Stream (cout);
			//cout<<"\n";
			//_popul->getIndividual(j)->print2Stream (cout);
			//cout<<"\n";
			//cout.flush ();
			mdist[j][i] = mdist[i][j] = dist.measure (tempInd, _popul->getIndividual(j));
			//cout<<i<<" : "<<j<<"\n";
			//cout<<temp<<"\n";
			//cout.flush ();
		}
	}

	return Q_OK;
}


QNichingOperator::QNichingOperator ()
: _popul(NULL) {
}

QNichingOperator::QNichingOperator (const QEvolutionaryAlgorithm* owner)
: QOperator(owner), _popul(NULL) {
}

QNichingOperator::QNichingOperator (const QNichingOperator& argNiching) : QOperator (argNiching), _popul(argNiching._popul) {
}

QNichingOperator::~QNichingOperator () {
	_popul = NULL;
}

QNichingOperator& QNichingOperator::operator= (const QNichingOperator& argNiching) {
	this->::QOperator::operator = (argNiching);
	_popul = argNiching._popul;
	return (*this);
}


int QNichingOperator::setPopulation (QPopulation* argPopul) {
	_popul = argPopul;
	return Q_OK;
}
