/*****************************************

ECWorkshop -- evolutionary computation
toolkit library

Web: http://qai.narod.ru/ecw/
Mailto: qai@mail.ru

Copyright (C) 2007 by Yury Tsoy


This program is free software; you can
redistribute it and/or modify it under the
terms of the GNU General Public License as
published by the Free Software Foundation;
either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that
it will be useful, but WITHOUT ANY WARRANTY;
without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR
PURPOSE. See the GNU General Public License
for more details.

You should have received a copy of the GNU
General Public License along with this
program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330,
Boston, MA  02111-1307  USA

*/

#include "qsteadypopulationsizer.h"

using namespace ECWorkshop;


///////////////////////////////////////////////////////////
//
//		QSteadyPopulationSizer class
//
///////////////////////////////////////////////////////////

QSteadyPopulationSizer::QSteadyPopulationSizer(void)
: _newPopSize(0), _direction(0), _step(1) {
}

QSteadyPopulationSizer::QSteadyPopulationSizer(const QEvolutionaryAlgorithm* owner)
: QPopulationSizer(owner), _newPopSize(0), _direction(0), _step(1) {
}

QSteadyPopulationSizer::QSteadyPopulationSizer (const QSteadyPopulationSizer& argSizer)
: QPopulationSizer(argSizer), _newPopSize(argSizer._newPopSize), _direction(argSizer._direction),
_step(argSizer._step) {
}

QSteadyPopulationSizer::~QSteadyPopulationSizer(void) {
}

QSteadyPopulationSizer& QSteadyPopulationSizer::operator= (const QSteadyPopulationSizer& argSizer) {
	this->QPopulationSizer::operator = (argSizer);

	_newPopSize = argSizer._newPopSize;
	_direction = argSizer._direction;
	_step = argSizer._step;

	return (*this);
}

int QSteadyPopulationSizer::setParameters (const QEAParameters& argParams) {
	int ret = this->QOperator::setParameters (argParams);

	_step = _params->sizingStep;
	_direction = _params->sizingDirection;
	_newPopSize = _params->populationSize + _direction * _step;
	return ret;
}

QPopulationSizer* QSteadyPopulationSizer::clone () const {
	return new QSteadyPopulationSizer (*this);
}

bool QSteadyPopulationSizer::populationReady () const {
	return (_popul->getSize() < _newPopSize)? false : true; 
}

int QSteadyPopulationSizer::operate () {
	if (_popul->getSize() > _newPopSize) {
		_popul->deleteIndividuals (_newPopSize, _popul->getSize() - _newPopSize);
	}

	// define new population size
	_newPopSize += _direction * _step;
	if (_newPopSize < _params->minPopSize) {
		_newPopSize = _params->minPopSize;
	} else if (_newPopSize > _params->maxPopSize) {
		_newPopSize = _params->maxPopSize;
	}
	return Q_OK;
}


