/*****************************************

ECWorkshop -- evolutionary computation
toolkit library

Web: http://qai.narod.ru/ecw/
Mailto: qai@mail.ru

Copyright (C) 2007 by Yury Tsoy


This program is free software; you can
redistribute it and/or modify it under the
terms of the GNU General Public License as
published by the Free Software Foundation;
either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that
it will be useful, but WITHOUT ANY WARRANTY;
without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR
PURPOSE. See the GNU General Public License
for more details.

You should have received a copy of the GNU
General Public License along with this
program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330,
Boston, MA  02111-1307  USA

*/

#include "qdeceptivefunction.h"

using namespace ECWorkshop;

///////////////////////////////////////////////////////////
//
//		QDeceptiveProblem class
//
///////////////////////////////////////////////////////////

QDeceptiveProblem::QDeceptiveProblem (void) {
}

QDeceptiveProblem::~QDeceptiveProblem (void) {
}

//
// Fitness minimization problem is considered, so the better
//  the individual the lower its fitness is!
//
int QDeceptiveProblem::evaluateIndividual (QIndividual* argInd) {
	__VECTOR_INT params;

	if (argInd->toVectorInt (params) == Q_OK) {
		double er=0;
		int i, gene, temp, tempGene;
		int g, b;	// g - current gene cursor,
					// b - current bit cursor in [g] gene

		for (i=0; i<argInd->getSize(); i++) {
			g = i/4;
			b = i%4;
			gene = 0;
			tempGene = params[g];
			if (tempGene & bitMask[b]) {
				gene |= 1;
			}

			temp = _params->chromosomeLength + b;	// find next (2-nd) "ugly" bit
			g += temp/4;
			b = temp%4;
			tempGene = params[g];
			if (tempGene & bitMask[b]) {
				gene |= 2;
			}

			temp = _params->chromosomeLength + b;	// find next (3-rd) "ugly" bit
			g += temp/4;
			b = temp%4;
			tempGene = params[g];
			if (tempGene & bitMask[b]) {
				gene |= 4;
			}

			temp = _params->chromosomeLength + b;	// find next (4-th) "ugly" bit
			g += temp/4;
			b = temp%4;
			tempGene = params[g];
			if (tempGene & bitMask[b]) {
				gene |= 8;
			}

			er += __FitnessWhitley[gene];
		}
		er = 30*_params->chromosomeLength - er;
		argInd->setFitness (er);

		return Q_OK;
	} else {
	        // shout aloud and ...
		cerr<<"\n\n[QDeceptiveProblem::evaluatePopulation] error:\n"
			<<"\tIncorrect encoding type or encoding parameters! (encoding ID = "<<argInd->getEncodingType()<<")\n\n";
	        // ... do nothing
		return Q_FAILURE;
	}

	return Q_OK;
}

int QDeceptiveProblem::evaluatePopulation (QPopulation* argPopul) {
	int i, popSize = argPopul->getSize ();

	for (i=0; i<popSize; i++) {
		evaluateIndividual (argPopul->getIndividual( i ));
	}

	return Q_OK;
}

int QDeceptiveProblem::getDefaultParameters (QEAParameters* argParams) {
	argParams->geneSize = 4;
	argParams->genePrecision = 1;
	argParams->geneOffset = 0;
	return Q_OK;
}

int QDeceptiveProblem::printProblemName (ostream& out) {
	out<<"4-ugly deceptive problem\n";
	return Q_OK;
}
