/*****************************************

ECWorkshop -- evolutionary computation
toolkit library

Web: http://qai.narod.ru/ecw/
Mailto: qai@mail.ru

Copyright (C) 2007 by Yury Tsoy


This program is free software; you can
redistribute it and/or modify it under the
terms of the GNU General Public License as
published by the Free Software Foundation;
either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that
it will be useful, but WITHOUT ANY WARRANTY;
without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR
PURPOSE. See the GNU General Public License
for more details.

You should have received a copy of the GNU
General Public License along with this
program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330,
Boston, MA  02111-1307  USA

*/

#include "qmixingtimeresearch.h"

using namespace ECWorkshop;

///////////////////////////////////////////////////////////
//
//		QMixingTimeResearch class
//
///////////////////////////////////////////////////////////

QMixingTimeResearch::QMixingTimeResearch (void) {
}

QMixingTimeResearch::~QMixingTimeResearch (void) {
}

//
// Fitness minimization problem is considered, so the better
//  the individual the lower its fitness is!
//
int QMixingTimeResearch::evaluateIndividual (QIndividual* argInd) {
	__VECTOR_INT params;

	if (argInd->toVectorInt (params)) {
		int i, size = (int)_params->chromosomeLength;
		double er = 0.0;
		double aspect = size / (double)_params->arity;
		__VECTOR_INT counts;

		counts.assign (_params->arity, 0);

		// calculate frequencies for different symbols in the given string
		for (i=0; i<size; i++) {
			counts[ params[i] ]++;
		}
		// calculate deviations for each symbol
		for (i=0; i<_params->arity; i++) {
			er += (aspect - counts[i]) * (aspect - counts[i]);
		}
		er /= _params->arity;

		argInd->setFitness (er);

		return Q_OK;

	} else {
	        // shout aloud and ...
		cerr<<"\n\n[QMixingTimeResearch::evaluateIndividual] error:\n"
			<<"\tIncorrect encoding type or encoding parameters! (encoding ID = "<<argInd->getEncodingType()<<")\n\n";
	        // ... do nothing
		return Q_FAILURE;
	}

	return Q_OK;
}

int QMixingTimeResearch::evaluatePopulation (QPopulation* argPopul) {
	if (argPopul->getEncodingType() == QET_STRING) {
		int i, popSize = argPopul->getSize ();

		for (i=0; i<popSize; i++) {
			evaluateIndividual (argPopul->getIndividual( i ));
		}

		return Q_OK;
	} else {
	        // shout aloud and ...
		cerr<<"\n\n[QMixingTimeResearch::evaluatePopulation] error:\n"
			<<"\tIncorrect encoding type! (encoding ID = "<<argPopul->getEncodingType()<<")\n\n";
	        // ... do nothing
		return Q_FAILURE;
	}
}

int QMixingTimeResearch::getDefaultParameters (QEAParameters* argParams) {
	argParams->geneSize = argParams->arity;
	argParams->genePrecision = 1;
	argParams->geneOffset = 0;
	argParams->populationSize = argParams->chromosomeLength = argParams->arity;
	return Q_OK;
}

int QMixingTimeResearch::printProblemName (ostream& out) {
	out<<"Mixing-time research\n";
	return Q_OK;
}
