/*****************************************

ECWorkshop -- evolutionary computation
toolkit library

Web: http://qai.narod.ru/ecw/
Mailto: qai@mail.ru

Copyright (C) 2007 by Yury Tsoy


This program is free software; you can
redistribute it and/or modify it under the
terms of the GNU General Public License as
published by the Free Software Foundation;
either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that
it will be useful, but WITHOUT ANY WARRANTY;
without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR
PURPOSE. See the GNU General Public License
for more details.

You should have received a copy of the GNU
General Public License along with this
program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330,
Boston, MA  02111-1307  USA

*/

#ifndef		Q_PROBLEM_H
#define		Q_PROBLEM_H

#include "../include/qec.h"
#include "../encodings/include.h"

namespace ECWorkshop {

///////////////////////////////////////////////////////////
//
//		QProblem class
//
///////////////////////////////////////////////////////////

class QProblem {
protected:
	QEAParameters *_params;	// the pointer is not [const] so
							// that EA parameters can be changed
							// according to the problem's properties

public:

	QProblem (void);
	virtual ~QProblem (void);

	virtual int setParameters (QEAParameters* argParams);

	// Sets [argParams] to default values according to the problem at hand
	// In most cases just ignore this function unless you need to set
	//  some EA's parameters that depend on the problem
	virtual int getDefaultParameters (QEAParameters* argParams) = 0;

	// int evaluateIndividual (QIndividual* argInd)
	// Evaluates given individual
	//	Returns Q_OK on success
	virtual int evaluateIndividual (QIndividual* argInd) = 0;

	// int evaluatePopulation (QPopulation* argPopul)
	// Evaluates all individuals in given population
	//	Returns Q_OK on success
	virtual int evaluatePopulation (QPopulation* argPopul) = 0;

	// int testIndividual (QIndividual* argInd)
	// Tests given individual (necessary for specific tasks, for example
	//	evolving ANN etc.)
	//	Returns Q_OK on success
	virtual int testIndividual (QIndividual* argInd) const;

	// int printProblemName (ostream& out)
	// Prints name of the problem
	//	Returns Q_OK on success
	virtual int printProblemName (ostream& out) = 0;
};

};	// namespace

#endif
