/*****************************************

ECWorkshop -- evolutionary computation
toolkit library

Web: http://qai.narod.ru/ecw/
Mailto: qai@mail.ru

Copyright (C) 2007 by Yury Tsoy


This program is free software; you can
redistribute it and/or modify it under the
terms of the GNU General Public License as
published by the Free Software Foundation;
either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that
it will be useful, but WITHOUT ANY WARRANTY;
without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR
PURPOSE. See the GNU General Public License
for more details.

You should have received a copy of the GNU
General Public License along with this
program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330,
Boston, MA  02111-1307  USA

*/

#include "qspherefunction.h"

using namespace ECWorkshop;

///////////////////////////////////////////////////////////
//
//		QSphereFunction class
//
///////////////////////////////////////////////////////////

QSphereFunction::QSphereFunction (void) {
}

QSphereFunction::~QSphereFunction (void) {
}

//
// Fitness minimization problem is considered, so the better
//  the individual the lower its fitness is!
//
int QSphereFunction::evaluateIndividual (QIndividual* argInd) {
	__VECTOR_DOUBLE params;

	if (argInd->toVectorDouble (params)) {
//		QRCIndividual* tempInd = (QRCIndividual*)argInd;
		int nParams = _params->chromosomeLength;
		double x;
		double er=0;

		for (int i=0; i<nParams; i++) {
//			cout<<tempInd->getGeneValue(i)<<" : "<<params[i]<<"\n";
			x = params[i];
			er += x * x;
		}
		argInd->setFitness (er);

		return Q_OK;

	} else {
	        // shout aloud and ...
		cerr<<"\n\n[QSphereFunction::evaluatePopulation] error:\n"
			<<"\tIncorrect encoding type or encoding parameters! (encoding ID = "<<argInd->getEncodingType()<<")\n\n";
	        // ... do nothing
		return Q_FAILURE;
	}

	return Q_OK;
}

int QSphereFunction::evaluatePopulation (QPopulation* argPopul) {
	int i, popSize = argPopul->getSize ();

	for (i=0; i<popSize; i++) {
		evaluateIndividual (argPopul->getIndividual( i ));
	}

	return Q_OK;
}

int QSphereFunction::getDefaultParameters (QEAParameters* argParams) {
	return Q_OK;
}

int QSphereFunction::printProblemName (ostream& out) {
	out<<"Sphere function\n";
	return Q_OK;
}
