/*****************************************

ECWorkshop -- evolutionary computation
toolkit library

Web: http://qai.narod.ru/ecw/
Mailto: qai@mail.ru

Copyright (C) 2007 by Yury Tsoy


This program is free software; you can
redistribute it and/or modify it under the
terms of the GNU General Public License as
published by the Free Software Foundation;
either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that
it will be useful, but WITHOUT ANY WARRANTY;
without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR
PURPOSE. See the GNU General Public License
for more details.

You should have received a copy of the GNU
General Public License along with this
program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330,
Boston, MA  02111-1307  USA

*/

#include "qsubmatrix.h"

#include <algorithm>
#include <direct.h>

#include <assert.h>

using namespace ECWorkshop;

QSubmatrixTask::QSubmatrixTask (void)
: _width(0), _height(0), _recHeight(0),
_totalWPlus(0), _totalWMinus(0), _totalWHarm(0) {
	readDataFromFile ("matrix.txt");
	_fout.open ("submatrix.log");
}

QSubmatrixTask::~QSubmatrixTask(void) {
	if (_fout.is_open ()) {
		_fout.close ();
	}
}

int QSubmatrixTask::evaluateIndividual (QIndividual* argInd) {
	double er=0;
	int size = (int)argInd->getSize();
	__VECTOR_INT con, dis;	// conjunction and disjunction of matrix rows, marked in by argParams
	int i, j, rows;
	int commonColumns, totalColumns;
	double sumStrWPlus, sumStrWMinus, sumStrWHarm;
	__VECTOR_INT params;
	// containers for coefficients recalculation
//	__VECTOR_DOUBLE modWPlus, modWMinus, modWHarm, modStrWPlus, modStrWMinus, modStrWHarm;
//	double modTotalWPlus, modTotalWMinus, modTotalWHarm;


	if (argInd->toVectorInt (params)) {
		// Temp variable. Should be replaced later on variable from config-file
		bool recalculateCoeffs = false;

		con.assign (_width, 1);
		dis.assign (_width, 0);
		sumStrWPlus = sumStrWMinus = sumStrWHarm = 0;

		if (!recalculateCoeffs) {

			// calculate sub-matrix statistics for the given chromosome
			if (_params->permutation == false) {
				for (i=0; i<size; i++) {
		//			cout<<"i = "<<i<<"\n";
					if (params[i] == 1) {
						for (j=0; j<_width; j++) {	// read matrix row
		//					cout<<j<<"\n";
							con[j] &= _matrix[ i ][ j ];
							dis[j] |= _matrix[ i ][ j ];
						}
						sumStrWPlus += _strWPlus[ i ];
						sumStrWMinus += _strWMinus[ i ];
						sumStrWHarm += _strWHarm[ i ];
					}
				}
				rows = (int) count (params.begin(), params.end(), 1);
			} else {	// if permutation encoding is used
				int row;

				for (i=0; i<size; i++) {
		//			cout<<"i = "<<i<<"\n";
					row = params[i];
					for (j=0; j<_width; j++) {	// read matrix row
	//					cout<<j<<"\n";
						con[j] &= _matrix[ row ][ j ];
						dis[j] |= _matrix[ row ][ j ];
					}
					sumStrWPlus += _strWPlus[ row ];
					sumStrWMinus += _strWMinus[ row ];
					sumStrWHarm += _strWHarm[ row ];
				}
				rows = size;
			}	// if (_params->permutation == false)

		} else {
			//
			// calculate coeffs
			//
			__VECTOR_UINT tests;	// array to store indices of rows
			__VECTOR_UINT units;	// array to store number of units in each test
			__VECTOR_DOUBLE coeffsW, coeffsC, coeffsH;
			double sumStrW, sumStrCost, sumStrHarm;
			double avgStrW = _totalWPlus / (double)_height,
					avgStrC = _totalWMinus / (double)_height,
					avgStrH = _totalWHarm / (double)_height,
					avgUnits;
			int countTemp, unitsJ;

			coeffsW.assign (_width, 1);
			coeffsC.assign (_width, 1);
			coeffsH.assign (_width, 1);
			if (!_params->permutation) {
				// if not permutation encoding

				// calculate units in each test
				units.assign (_height, 0);
				avgUnits = 0;
				for (i=0; i<_height; i++) {
					for (j=0; j<_width; j++) {
						if (_matrix[i][j] == 1) {
							units[i]++;
						}
					}
					avgUnits += units[i];
				}	// for (i=0; i<_height; i++)
				avgUnits /= _height;

				for (j=0; j<_width; j++) {	// for each feature

					// define tests in IUDT that contain the j-th feature
					tests.clear ();
					sumStrW = sumStrCost = sumStrHarm = 0;
					countTemp = unitsJ = 0;
					for (i=0; i<size; i++) {
						if (params[i] == 1) {
							if (_matrix [ i ][ j ] == 1) {
								tests.push_back (i);
								sumStrW		+= _strWPlus[i];
								sumStrCost	+= _strWMinus[i];
								sumStrHarm	+= _strWHarm[i];
								countTemp++;
								unitsJ += units[i];
							}
						}
					}

					// calculate coeffs
					coeffsW[j] = (sumStrW / (double)countTemp*avgStrW) * (avgUnits*countTemp / (double)unitsJ);
					coeffsC[j] = (sumStrCost / (double)countTemp*avgStrC) * (avgUnits*countTemp / (double)unitsJ);
					coeffsH[j] = (sumStrHarm / (double)countTemp*avgStrH) * (avgUnits*countTemp / (double)unitsJ);

				}	// for (j=0; j<_width; j++)

				// TODO
				// calculate objective function
				for (i=0; i<size; i++) {
		//			cout<<"i = "<<i<<"\n";
					if (params[i] == 1) {
						for (j=0; j<_width; j++) {	// read matrix row
							con[j] &= _matrix[ i ][ j ];
							dis[j] |= _matrix[ i ][ j ];

							if (_matrix[ i ][ j ] == 1) {
								sumStrWPlus		+= coeffsW[j] * _wPlus[j];
								sumStrWMinus	+= coeffsC[j] * _wMinus[j];
								sumStrWHarm		+= coeffsH[j] * _wHarm[j];
							}
						}
					}
				}	// for (i=0; i<size; i++)
				rows = (int) count (params.begin(), params.end(), 1);
			} else {
				rows = size;
			}	// if (_params->permutation)
		}	// if (!recalculateCoeffs)

		commonColumns = (int) count (con.begin(), con.end(), 1);
		totalColumns = (int) count (dis.begin(), dis.end(), 1);

		//	For fitness
		// sumStrWPlus -> max
		// commonColumns -> max
		// sumStrWMinus -> min
		// totalColumns -> min
		// rows -> min
		//	For error
		// sumStrWPlus -> min
		// commonColumns -> min
		// sumStrWMinus -> max
		// totalColumns -> max
		// rows -> max
		if (_totalWHarm != 0) {
			er =	+ 100*(rows-_recHeight)*(rows-_recHeight)
					+ 40.0*((double)_width - commonColumns)/((double)_width)
					+ 30.0*totalColumns/(double)(_width)
					+ 15.0*(_totalWPlus - sumStrWPlus)/(double)_totalWPlus
					+ 10.0*(sumStrWMinus/(double)_totalWMinus)
					+ 5.0*(sumStrWHarm/(double)_totalWHarm);
		} else {
			er =	+ 100*(rows-_recHeight)*(rows-_recHeight)
					+ 40.0*((double)_width - commonColumns)/((double)_width)
					+ 30.0*totalColumns/(double)(_width)
					+ 20.0*(_totalWPlus - sumStrWPlus)/(double)_totalWPlus
					+ 10.0*(sumStrWMinus/(double)_totalWMinus);
		}

		argInd->setFitness (er);
		return Q_OK;

	} else {
		cerr<<"\n\n[QSubmatrixTask::evaluatePopulation] error:\n"
			<<"\tIncorrect encoding type or encoding parameters! (encoding ID = "<<argInd->getEncodingType()<<")\n\n";
		return Q_FAILURE;
	}

	return Q_OK;
}

int QSubmatrixTask::evaluatePopulation (QPopulation* argPopul) {
	int i, popSize = argPopul->getSize ();

	for (i=0; i<popSize; i++) {
		evaluateIndividual (argPopul->getIndividual( i ));
	}

	return Q_OK;
}

////////////////////////////////////////////////////////
//
// Problem specific methods
//

int QSubmatrixTask::getParametersNumber () {
	return (int)_matrix.size();
}

int QSubmatrixTask::readDataFromFile (const char* argFilename) {
	// switch to the default program directory
	char _path [_MAX_PATH];
	int curdrive = _getdrive();
	_getdcwd (curdrive, _path, _MAX_PATH);
	chdir (_path);

	// read matrix
	int i, j;
	ifstream fin (argFilename);

	if (fin.is_open ()) {

		// read task data
		//	file format:
		//	- dimensions (rows, columns)
		//	- matrix
		//	- number of types of weights
		//	- weights vectors
		//	- recommended resulting submatrix height
		fin>>_height>>_width;
		_matrix.resize (_height);
		_strWPlus.resize (_height);
		_strWMinus.resize (_height);
		_strWHarm.resize (_height);
		_wPlus.resize (_width);
		_wMinus.resize (_width);
		_wHarm.resize (_width);

		// read matrix
		for (i=0; i<_height; i++) {
			_matrix[i].resize (_width);
			for (j=0; j<_width; j++) {
				fin>>_matrix[i][j];
			}
		}

		// read maximisable weights
		for (i=0; i<_width; i++) {
			fin>>_wPlus[i];
		}

		// read minimisable weights
		for (i=0; i<_width; i++) {
			fin>>_wMinus[i];
		}

		// read harm weights (to minimize)
		for (i=0; i<_width; i++) {
			fin>>_wHarm[i];
		}

		fin>>_recHeight;
		fin.close ();

		calculateWeights ();

		_fout.flush ();

		return Q_OK;

	} else {

		cerr<<"[QSubmatrixTask::readDataFromFile] error:\n"
			<<"\tCan't open input file ("<<argFilename<<") for reading\n";
		exit (0);
		return Q_FAILURE;

	}	// if (fin.is_open ()
}

int QSubmatrixTask::setData (const __TABLE_INT& argData, const __TABLE_DOUBLE& argWeights) {
//     MessageBox (NULL, "QSubmatrixTask::setData", "QSubmatrixTask::setData", MB_OK);
    if (argData.size () > 0) {
	   int i, j;

	   _height = (int)argData.size();
       _width = (int)argData[0].size();

       _matrix.resize (_height);
	   _strWPlus.resize (_height);
	   _strWMinus.resize (_height);
	   _strWHarm.resize (_height);
	   _wPlus.resize (_width);
	   _wMinus.resize (_width);
	   _wHarm.resize (_width);

	   // read matrix
	   _fout<<"The submatrix formation task (construction of subset of the irredundant diagostic tests):\n";
	   for (i=0; i<_height; i++) {
	       _matrix[i].resize (_width);
		   for (j=0; j<_width; j++) {
			   _matrix[i][j] = argData[i][j];
			   _fout<<_matrix[i][j]<<" ";
           }
		   _fout<<"\n";
       }
	   _fout<<"\n";
	   _fout.flush ();

	   // read weights
	   for (i=0; i<_width; i++) {
	       _wPlus[i] = argWeights[0][i];
       }

	   // read costs
	   for (i=0; i<_width; i++) {
		   _wMinus[i] = argWeights[1][i];
       }

	   // read harms (to minimize)
	   for (i=0; i<_width; i++) {
	       _wHarm[i] = argWeights[2][i];
       }
       _recHeight = (int)argWeights[3][0];

       calculateWeights ();

       return Q_OK;
    } else {
       return Q_FAILURE;
    }
}

//*********************************************************
//	calculate weights of rows
int QSubmatrixTask::calculateWeights () {
	int i, j;

	_totalWPlus = _totalWMinus = _totalWHarm = 0;
	_strWPlus.assign (_height, 0);
	_strWMinus.assign (_height, 0);
	_strWHarm.assign (_height, 0);
	for (i=0; i<_height; i++) {
		//strWPlus[i] = 0;
		//strWMinus[i] = 0;
		for (j=0; j<_width; j++) {
			_strWPlus[i] += _matrix[i][j] * _wPlus[j];
			_strWMinus[i] += _matrix[i][j] * _wMinus[j];
			_strWHarm[i] += _matrix[i][j] * _wHarm[j];
		}
		_totalWPlus += _strWPlus[i];
		_totalWMinus += _strWMinus[i];
		_totalWHarm += _strWHarm[i];
//		_fout<<strWPlus[i]<<"\t"<<strWMinus[i]<<"\t"<<strWHarm[i]<<"\n";
	}

	return Q_OK;
}

int QSubmatrixTask::printSolution (const QIndividual* argInd) const {
	int i, j, size = argInd->getSize(), rows;
	double solWeight, solCost, solHarm;
	__TABLE_INT subM;
	__VECTOR_INT params;

	if (argInd->toVectorInt (params)) {
		_fout<<"\n\nCurrent solution:\n"
			<<"---------------------\n";

		rows = 0;
		solWeight = solCost = solHarm = 0.0;
		for (i=0; i<size; i++) {
			if (params[i] == 1) {
				// print correspondent row of the matrix
				_fout<<i<<":\t";
				for (j=0; j<_width; j++) {
					_fout<<_matrix[i][j]<<" ";
				}
				_fout<<"\n";
				subM.push_back (_matrix[i]);
				solWeight += _strWPlus [i];
				solCost += _strWMinus [i];
				solHarm += _strWHarm [i];
				rows++;
			}
		}	// for (i=0; i<size; i++)
		_fout<<"\nNumber of rows: "<<rows<<"\n";

		// calculate total and common columns in resulting submatrix
		__VECTOR_INT con, dis;
		int totalColumns, commonColumns;
		con.assign (_width, 1);
		dis.assign (_width, 0);
		for (i=0; i<(int)subM.size(); i++) {
			for (j=0; j<_width; j++) {
				con[j] &= subM[ i ][ j ];
				dis[j] |= subM[ i ][ j ];
			}
		}
		totalColumns = (int) count (dis.begin(), dis.end(), 1);
		commonColumns = (int) count (con.begin(), con.end(), 1);
		_fout<<"Total columns:\t"<<totalColumns<<"\n";
		_fout<<"Common columns:\t"<<commonColumns<<"\n";

		// calculate features statistics
		__VECTOR_INT feats;
		feats.assign (_width, 0);	// fill with zeros
		for (i=0; i<(int)subM.size(); i++) {
			for (j=0; j<_width; j++) {
				if (subM[i][j] == 1) {
					feats[j] ++;
				}
			}
		}

		// print feats
		_fout<<"\nFeatures count distribution:\n";
		for (j=0; j<_width; j++) {
			if (feats[j] != 0) {
				_fout<<(j+1)<<":\t"<<feats[j]<<"\n";
			}
		}
		_fout<<"\nTotal weight | cost | harm:\t"<<solWeight<<"\t"<<solCost<<"\t"<<solHarm<<"\n";

		double firstWeight = 0.0, firstCost = 0.0, firstHarm = 0.0;
		for (i=0; i<(int)subM.size(); i++) {
			firstWeight += _strWPlus[i];
			firstCost += _strWMinus[i];
			firstHarm += _strWHarm[i];
		}

		_fout<<"\nTotal weight | cost | harm for the first "<<(int)subM.size()<<" rows:\t"<<firstWeight<<"\t"<<firstCost<<"\t"<<firstHarm<<"\n";
	}

	return Q_OK;
}

int QSubmatrixTask::getDefaultParameters (QEAParameters* argParams) {
	if (argParams->permutation == true) {
		argParams->arity = _height;
		argParams->chromosomeLength = _recHeight;
		argParams->mRate = 0.7 / _recHeight;
	} else {
		argParams->chromosomeLength = _height;
		argParams->mRate = 0.7 / (double)getParametersNumber ();
	}
	argParams->unitsRatio = (double)_recHeight / (double)_height;
	return Q_OK;
}

int QSubmatrixTask::printProblemName (ostream& out) {
	out<<"Submatrix problem\n";
	return Q_OK;
}
